"""Example molecular structures for testing and demonstration purposes."""

from .get_example_files import acrolein, all_examples, benzene, co, co2, furan, h2o, o2, prismane, pyridine

__all__ = [
    'acrolein',
    'all_examples',
    'benzene',
    'co',
    'co2',
    'furan',
    'h2o',
    'o2',
    'prismane',
    'pyridine',
]
